/**********************************************************************
Copyright (c) 2009-2010 Alexander Kerner. All rights reserved.
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
 ***********************************************************************/

package net.sf.jcmdlineparser.helpprinter;

import java.io.OutputStream;
import java.io.PrintWriter;
import java.util.Formatter;
import java.util.Set;

import net.sf.jcmdlineparser.Utils;
import net.sf.jcmdlineparser.options.AbstractOption;

/**
 * <p>
 * A simple implementation for {@link HelpPrinter}.
 * </p>
 * 
 * @author <a href="mailto:alex.kerner.24@googlemail.com">Alexander Kerner</a>
 * @version 2010-08-10
 */
public class SimpleHelpPrinter implements HelpPrinter {

	public synchronized void printHelp(Set<AbstractOption<?>> options,
			String usageString) {
		printHelp(options, usageString, System.out);
	}

	public synchronized void printHelp(Set<AbstractOption<?>> options,
			String usageString, OutputStream out) {
		PrintWriter writer = new PrintWriter(out);
		Formatter formatter = new Formatter(writer);
		writer.println(usageString);
		writer.println("options:");
		// writer.println();
		writer.println("required:");
		formatter.format("\t%-6s %-18s %-10s %-50s", "short", "long", "param",
				"description");
		writer.println();
		for (AbstractOption<?> o : options) {
			if (o.isRequired()) {
				String l = o.getKeyLong();
				Character c = o.getKeyShort();
				String d = o.getDescription();
				Class<?> r = o.getReturnType();
				String r2 = r.getSimpleName();
				formatter.format("\t%-6s %-18s %-10s %-50s",
						"-" + c.toString(), "--" + l, r2, d);
				writer.println();
			}
		}
		writer.println();
		writer.println("mandatory:");
		// writer.print("\t");
		formatter.format("\t%-6s %-18s %-10s %-50s", "short", "long", "param",
				"description");
		writer.println();
		for (AbstractOption<?> o : options) {
			if (!o.isRequired()) {
				String l = o.getKeyLong();
				Character c = o.getKeyShort();
				String d = o.getDescription();
				Class<?> r = o.getReturnType();
				String r2 = r.getSimpleName();
				// writer.print("\t");
				formatter.format("\t%-6s %-18s %-10s %-10s\t",
						"-" + c.toString(), "--" + l, r2, d);
				writer.println();
			}
		}
		writer.println();
		formatter.flush();
		Utils.closeProperly(writer);
	}
}
