/**********************************************************************
Copyright (c) 2009-2010 Alexander Kerner. All rights reserved.
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
 *******************************************************************/

package net.sf.jcmdlineparser.options;


/**
 * <p>
 * An abstract extension for {@link AbstractOption}. This command line option
 * can have only one value.
 * </p>
 * 
 * @author <a href="mailto:alex.kerner.24@googlemail.com">Alexander Kerner</a>
 * @version 2010-08-10
 * 
 */
public abstract class AbstractSingleOption<T> extends AbstractOption<T> {

	// lastVisit 2010-08-11

	private volatile T value = null;

	/**
	 * TODO
	 */
	protected AbstractSingleOption(Class<T> returnType, char identifierShort,
			String identifierLong, String description, boolean required) {
		super(returnType, identifierShort, identifierLong, description,
				required);
	}

	/**
	 * TODO
	 */
	protected AbstractSingleOption(Class<T> returnType, char identifierShort,
			String identifierLong, String description, boolean required,
			T initValue) {
		super(returnType, identifierShort, identifierLong, description,
				required);
		setValue(initValue);
	}

	/**
	 * <p>
	 * Get value.
	 * </p>
	 * 
	 * @throws OptionNotSetException
	 *             if this {@link AbstractOption} was not provided on the
	 *             command line
	 */
	public final T getValue() {
		if (this.isSet())
			return value;
		throw new OptionNotSetException("option \"" + this + "\" not set");
	}

	/**
	 * <p>
	 * Set value.
	 * </p>
	 * 
	 * @param values
	 */
	public final void setValue(T value) {
		this.value = value;
		this.setSet(true);
	}

}
