/**********************************************************************
Copyright (c) 2009-2010 Alexander Kerner. All rights reserved.
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
 *********************************************************************/

package net.sf.jcmdlineparser.options;

import java.io.File;
import java.util.Collection;
import java.util.LinkedHashSet;
import java.util.Set;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * <p>
 * Helper class to build options.
 * </p>
 * 
 * @author <a href="mailto:alex.kerner.24@googlemail.com">Alexander Kerner</a>
 * @version 2010-08-10
 * 
 * @see AbstractSingleOption
 * @see AbstractMultiOption
 * 
 */
public class OptionBuilder {

	private final static Logger log = LoggerFactory.getLogger(OptionBuilder.class);

	private volatile String longIdentifier;

	private volatile char shortIdentifier;

	private volatile String description = "<none>";

	private volatile boolean required = false;

	/**
	 * TODO
	 */
	protected final Set<AbstractOption<?>> clashOptions = new LinkedHashSet<AbstractOption<?>>();
	
	/**
	 * TODO
	 */
	protected final Set<AbstractOption<?>> requiredOptions = new LinkedHashSet<AbstractOption<?>>();

	/**
	 * TODO
	 */
	public synchronized OptionBuilder addClashOption(AbstractOption<?> clashOption) {
		if (clashOptions.add(clashOption)) {
			// all good
		} else {
			log.debug("clashoption \"" + clashOption
					+ "\" was already registered");
		}
		return this;
	}
	
	/**
	 * TODO
	 */
	public synchronized OptionBuilder addRequiredOption(AbstractOption<?> requiredOption) {
		if(requiredOption == null)
			throw new NullPointerException();
		if (requiredOptions.add(requiredOption)) {
			// all good
		} else {
			log.debug("requiredOption \"" + requiredOption
					+ "\" was already registered");
		}
		return this;
	}

	// Getter / Setter //

	/**
	 * TODO
	 */
	public synchronized String getLongIdentifier() {
		return longIdentifier;
	}

	/**
	 * TODO
	 */
	public synchronized char getShortIdentifier() {
		return shortIdentifier;
	}

	/**
	 * TODO
	 */
	public synchronized String getDescription() {
		return description;
	}

	/**
	 * TODO
	 */
	public synchronized boolean isRequired() {
		return required;
	}

	/**
	 * TODO
	 */
	public synchronized OptionBuilder setLongIdentifier(String longIdentifier) {
		this.longIdentifier = longIdentifier;
		return this;
	}

	/**
	 * TODO
	 */
	public synchronized OptionBuilder setShortIdentifier(char shortIdentifier) {
		this.shortIdentifier = shortIdentifier;
		return this;
	}

	/**
	 * TODO
	 */
	public synchronized OptionBuilder setDescription(String description) {
		this.description = description;
		return this;
	}

	/**
	 * TODO
	 */
	public synchronized OptionBuilder setRequired(boolean required) {
		this.required = required;
		return this;
	}

	// Factory //

	/**
	 * TODO
	 */
	public synchronized StringSingleOption newStringSingleOption() throws BuildException {
		assignIdentifiers();
		final StringSingleOption opt = new StringSingleOption(String.class, shortIdentifier,
				longIdentifier, description, required);
		for(AbstractOption<?> o : clashOptions){
			opt.addClashOption(o);
		}
		for(AbstractOption<?> o : requiredOptions){
			opt.addRequiredOption(o);
		}
		return opt;
	}

	/**
	 * TODO
	 */
	public synchronized StringSingleOption newStringSingleOption(String init) throws BuildException {
		assignIdentifiers();
		final StringSingleOption opt = new StringSingleOption(String.class, shortIdentifier,
				longIdentifier, description, required, init);
		for(AbstractOption<?> o : clashOptions){
			opt.addClashOption(o);
		}
		for(AbstractOption<?> o : requiredOptions){
			opt.addRequiredOption(o);
		}
		return opt;
	}

	/**
	 * TODO
	 */
	public synchronized StringMultiOption newStringMultiOption() throws BuildException {
		assignIdentifiers();
		final StringMultiOption opt = new StringMultiOption(String.class, shortIdentifier,
				longIdentifier, description, required);
		for(AbstractOption<?> o : clashOptions){
			opt.addClashOption(o);
		}
		for(AbstractOption<?> o : requiredOptions){
			opt.addRequiredOption(o);
		}
		return opt;
	}
	
	/**
	 * TODO
	 */
	public synchronized StringMultiOption newStringMultiOption(Collection<? extends String> init) throws BuildException {
		assignIdentifiers();
		final StringMultiOption opt = new StringMultiOption(String.class, shortIdentifier,
				longIdentifier, description, required, init);
		for(AbstractOption<?> o : clashOptions){
			opt.addClashOption(o);
		}
		for(AbstractOption<?> o : requiredOptions){
			opt.addRequiredOption(o);
		}
		return opt;
	}
	
	/**
	 * TODO
	 */
	public synchronized BooleanOption newBooleanOption() throws BuildException {
		assignIdentifiers();
		final BooleanOption opt = new BooleanOption(Boolean.class, shortIdentifier,
				longIdentifier, description, required);
		for(AbstractOption<?> o : clashOptions){
			opt.addClashOption(o);
		}
		for(AbstractOption<?> o : requiredOptions){
			opt.addRequiredOption(o);
		}
		return opt;
	}

	/**
	 * TODO
	 */
	public synchronized BooleanOption newBooleanOption(Boolean init) throws BuildException {
		assignIdentifiers();
		final BooleanOption opt = new BooleanOption(Boolean.class, shortIdentifier,
				longIdentifier, description, required, init);
		for(AbstractOption<?> o : clashOptions){
			opt.addClashOption(o);
		}
		for(AbstractOption<?> o : requiredOptions){
			opt.addRequiredOption(o);
		}
		return opt;
	}

	/**
	 * TODO
	 */
	public synchronized VoidOption newVoidOption() throws BuildException {
		assignIdentifiers();
		final VoidOption opt = new VoidOption(Void.class, shortIdentifier,
				longIdentifier, description, required);
		for(AbstractOption<?> o : clashOptions){
			opt.addClashOption(o);
		}
		for(AbstractOption<?> o : requiredOptions){
			opt.addRequiredOption(o);
		}
		return opt;
	}

	/**
	 * TODO
	 */
	public synchronized FileSingleOption newFileSingleOption() throws BuildException {
		assignIdentifiers();
		final FileSingleOption opt = new FileSingleOption(File.class, shortIdentifier,
				longIdentifier, description, required);
		for(AbstractOption<?> o : clashOptions){
			opt.addClashOption(o);
		}
		for(AbstractOption<?> o : requiredOptions){
			opt.addRequiredOption(o);
		}
		return opt;
	}
	
	/**
	 * TODO
	 */
	public synchronized FileSingleOption newFileSingleOption(File initValue) throws BuildException {
		assignIdentifiers();
		final FileSingleOption opt = new FileSingleOption(File.class, shortIdentifier,
				longIdentifier, description, required, initValue);
		for(AbstractOption<?> o : clashOptions){
			opt.addClashOption(o);
		}
		for(AbstractOption<?> o : requiredOptions){
			opt.addRequiredOption(o);
		}
		return opt;
	}

	/**
	 * TODO
	 */
	public synchronized FileMultiOption newFileMultiOption(Collection<? extends File> initValue) throws BuildException {
		assignIdentifiers();
		final FileMultiOption opt = new FileMultiOption(File.class, shortIdentifier,
				longIdentifier, description, required, initValue);
		for(AbstractOption<?> o : clashOptions){
			opt.addClashOption(o);
		}
		for(AbstractOption<?> o : requiredOptions){
			opt.addRequiredOption(o);
		}
		return opt;
	}
	
	/**
	 * TODO
	 */
	public synchronized FileMultiOption newFileMultiOption() throws BuildException {
		assignIdentifiers();
		final FileMultiOption opt = new FileMultiOption(File.class, shortIdentifier,
				longIdentifier, description, required);
		for(AbstractOption<?> o : clashOptions){
			opt.addClashOption(o);
		}
		for(AbstractOption<?> o : requiredOptions){
			opt.addRequiredOption(o);
		}
		return opt;
	}

	/**
	 * TODO
	 */
	public synchronized IntegerSingleOption newIntegerSingleOption() throws BuildException {
		assignIdentifiers();
		final IntegerSingleOption opt = new IntegerSingleOption(Integer.class, shortIdentifier,
				longIdentifier, description, required);
		for(AbstractOption<?> o : clashOptions){
			opt.addClashOption(o);
		}
		for(AbstractOption<?> o : requiredOptions){
			opt.addRequiredOption(o);
		}
		return opt;
	}

	/**
	 * TODO
	 */
	public synchronized IntegerSingleOption newIntegerSingleOption(Integer init) throws BuildException {
		assignIdentifiers();
		final IntegerSingleOption opt = new IntegerSingleOption(Integer.class, shortIdentifier,
				longIdentifier, description, required, init);
		for(AbstractOption<?> o : clashOptions){
			opt.addClashOption(o);
		}
		for(AbstractOption<?> o : requiredOptions){
			opt.addRequiredOption(o);
		}
		return opt;
	}

	private void assignIdentifiers() throws BuildException {
		if (shortIdentifier == '\u0000' && longIdentifier == null)
			throw new BuildException(
					"must provide at least one identifier for option");
		if (longIdentifier == null)
			longIdentifier = Character.toString(shortIdentifier);
		if (shortIdentifier == '\u0000')
			shortIdentifier = longIdentifier.charAt(0);
	}

}
