/**********************************************************************
Copyright (c) 2009-2010 Alexander Kerner. All rights reserved.
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
 *********************************************************************/

package net.sf.jcmdlineparser.options;

import java.util.LinkedHashSet;
import java.util.Set;

/**
 * 
 * 
 * TODO description
 * 
 * <p>
 * <b>Example:</b>
 * 
 * <pre>
 * TODO example
 * </pre>
 * 
 * </p>
 * 
 * @author <a href="mailto:alex.kerner.24@googlemail.com">Alexander Kerner</a>
 * @version 2010-10-19
 * 
 */
public class OptionUtils {

	/**
	 * 
	 */
	public final static String shortPrefix = "-";
	/**
	 * 
	 */
	public final static String longPrefix = shortPrefix + shortPrefix;

	private OptionUtils() {

	}

	public static boolean optionKnown(Set<AbstractOption<?>> options,
			String ident) {
		ident = ident.replaceAll(shortPrefix, "");
		for (AbstractOption<?> o : options) {
			if (o.getKeyLong().equals(ident)
					|| (ident.length() == 1 && o.getKeyShort() == ident
							.charAt(0)))
				return true;
		}
		return false;
	}

	/**
	 * 
	 * <p>
	 * TODO description
	 * </p>
	 * 
	 * <p>
	 * TODO example
	 * </p>
	 * 
	 * @param options
	 * @param ident
	 * @return
	 * @throws NoSuchOptionException
	 */
	public static AbstractOption<?> getOptionForKey(
			Set<AbstractOption<?>> options, String ident)
			throws NoSuchOptionException {
		ident = ident.replaceAll(shortPrefix, "");
		if (ident == null || ident.length() < 1)
			throw new NullPointerException();
		if (ident.length() == 1)
			return getOptionForShortKey(options, ident.charAt(0));
		return getOptionForLongKey(options, ident);
	}

	private static AbstractOption<?> getOptionForShortKey(
			Set<AbstractOption<?>> options, char c)
			throws NoSuchOptionException {
		for (AbstractOption<?> o : options) {
			if (o.getKeyShort() == c)
				return o;
		}
		throw new NoSuchOptionException("option \"" + c + "\" not known");
	}

	/**
	 * 
	 * <p>
	 * TODO description
	 * </p>
	 * 
	 * <p>
	 * TODO example
	 * </p>
	 * 
	 * @param options
	 * @param ident
	 * @return
	 * @throws NoSuchOptionException
	 */
	private static AbstractOption<?> getOptionForLongKey(
			Set<AbstractOption<?>> options, String ident)
			throws NoSuchOptionException {
		for (AbstractOption<?> o : options) {
			if (o.getKeyLong().equals(ident))
				return o;
		}
		throw new NoSuchOptionException("option \"" + ident + "\" not known");
	}

	/**
	 * 
	 * <p>
	 * TODO description
	 * </p>
	 * 
	 * <p>
	 * TODO example
	 * </p>
	 * 
	 * @param options
	 * @throws OptionClashException
	 */
	public static void checkOptionClashes(
			LinkedHashSet<AbstractOption<?>> options)
			throws OptionClashException {
		for (AbstractOption<?> o1 : options) {
			for (AbstractOption<?> o2 : options) {
				if (o1.isSet() && o2.isSet()
						&& o1.getClashOptions().contains(o2))
					throw new OptionClashException("options \"" + o1
							+ "\" and \"" + o2 + "\" cannot coexist!");
			}
		}
	}

	/**
	 * 
	 * <p>
	 * TODO description
	 * </p>
	 * 
	 * <p>
	 * TODO example
	 * </p>
	 * 
	 * @param options
	 * @throws MissingRequiredOptionException
	 */
	public static void checkRequiredDependency(
			LinkedHashSet<AbstractOption<?>> options)
			throws MissingRequiredOptionException {
		for (AbstractOption<?> o1 : options) {
			Set<AbstractOption<?>> req = o1.getRequiredOptions();
			for (AbstractOption<?> o2 : req) {
				
				// check only requirements if option itself is set
				if (o1.isSet()) {
					if (o2.isSet()) {
						// all good
					} else
						throw new MissingRequiredOptionException("option \""
								+ o1 + "\" requires option \"" + o2
								+ "\" which is not set!");
				}
			}
		}
	}

	/**
	 * 
	 * <p>
	 * TODO description
	 * </p>
	 * 
	 * <p>
	 * TODO example
	 * </p>
	 * 
	 * @param options
	 * @throws MissingRequiredOptionException
	 */
	public static void checkRequiredSimple(
			LinkedHashSet<AbstractOption<?>> options)
			throws MissingRequiredOptionException {
		for (AbstractOption<?> o1 : options) {
			if (o1.isRequired()) {
				if (o1.isSet()) {
					// all good
				} else
					throw new MissingRequiredOptionException("option \"" + o1
							+ "\" is required");
			}
		}
	}
}
